#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Version=Beta
#AutoIt3Wrapper_Icon=EcoServerLauncher.ico
#AutoIt3Wrapper_Outfile=EcoServerLauncher.exe
#AutoIt3Wrapper_UseX64=n
#AutoIt3Wrapper_Change2CUI=y
#AutoIt3Wrapper_Res_Comment=Eco Server Launcher by C.Hackl
#AutoIt3Wrapper_Res_Description=Eco Server Launcher by C.Hackl
#AutoIt3Wrapper_Res_Fileversion=0.1
#AutoIt3Wrapper_Res_Language=1033
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
;Use Console-Mode
;Disable Compression (Some Virus-Scanner have problems with compressed EXEs)
#AutoIt3Wrapper_Run_Debug_Mode=n
#cs ----------------------------------------------------------------------------

Eco Server Launcher
* Checks if Eco-Server is running (Checking if PID is still running)
* Checks if Eco-Server is hanging (Checking the internal WebAPI info)
* Checks Internet Connection (by requesting the public IP)
* Checks Public IP (by comparing it to the last IP)
* Can handle DynDns (Updates via URL without Login)
* Informs you via mail if server may have crashed or other problems (Own SMTP server or SMTP with SSL-Auth needed)

04-2018 by C.Hackl

#ce ----------------------------------------------------------------------------

#include <Inet.au3>
#include <Date.au3>
#include "Json.au3"
#include "smtpmailer.au3"
#include <TrayConstants.au3>

Global $pid = -1
Global $timerstartup = 0
Global $infocheck = False
Global $interval_count = 0
Global $interval_count_low = 0
Global $publicIP = ""

;Define Global Parameters (VB Compare Dim ... as ...)
Global $ecoserverpath
Global $ecoserverexe
Global $parameters
Global $ecoserverurl
Global $logfile
Global $logfilepath
Global $interval_sec
Global $interval_ms
Global $disablecheck
Global $ecoserverportweb
Global $ecoserverportgame
Global $backupmove
Global $backuporigin
Global $inisavedata
Global $ecoserverloadfinish
Global $lockcmd

;Trays
Global $trayCMDlock
Global $trayCheck
Global $trayECOStart
Global $trayECOStop
Global $trayEmergencyExit

Func init()
	CheckExit()
	CreateTrays()
	;ResetSavedata
	$inisavedata = @ScriptDir & "\param\savedata.ini"
	IniWrite($inisavedata, "SAVEDATA", "LastEcoServerPID", -1)
	$pid = -1
	Sleep(1000)
	;Load Config
	initconfig()
	;See any Launcher running?
	$plist = ProcessList(@ScriptName)
	$found = False
	For $i = 1 to $plist[0][0]
		If @AutoItPID <> $plist[$i][1] Then
			$found = True
			;ProcessClose($plist[$i][1])
			LogMessage(0,"Seeing other Eco-Server-Launcher. Exiting.","")
			Sleep(5000)
			Exit
		EndIf
	Next
	;Say Hello
	LogMessage(1,"Service started on PID " & @AutoItPID,"")
	$timerstartup = TimerInit()
EndFunc

Func CreateTrays()
	Opt("TrayMenuMode", 1 + 2)
	Opt("TrayAutoPause",0)
	Opt("TrayOnEventMode",1)
	TraySetIcon(@ScriptDir & "\EcoServerLauncher.ico",0)
	TraySetPauseIcon(@ScriptDir & "\EcoServerLauncher.ico",0)
	$trayCMDlock = CreateTrayEvent("Lock Server CMD Window", "TrayEventCMDlock")
	$trayCheck = CreateTrayEvent("Disable All Launcher Checks", "TrayEventCheck")
	TrayCreateItem("")
	CreateTrayEvent("Startup Eco Server", "TrayEventStartup")
	CreateTrayEvent("Quit Eco Server", "TrayEventShutdown")
	TrayCreateItem("")
	CreateTrayEvent("Send Testmail if SMTP is enabled", "TrayEventTestmail")
	TrayCreateItem("")
	CreateTrayEvent("EMERGENCY EXIT", "TrayEventEmergency")
	TrayCreateItem("")
	CreateTrayEvent("Quit Launcher", "TrayEventQuit")
	CreateTrayEvent("Quit Server and Launcher", "TrayEventQuitAll")
	;TraySetState($TRAY_ICONSTATE_SHOW)
EndFunc

Func initconfig()
	;Set INI Path
	$ini = @ScriptDir & "\param\config.ini"
	;Load Parameters from config
	$ecoserverpath = IniRead($ini,"ESLConfig","ecoserverpath","")
	$ecoserverexe = IniRead($ini,"ESLConfig","ecoserverexe","EcoServer.exe")
	$parameters = IniRead($ini,"ESLConfig","parameters","")
	$ecoserverportgame = Int(IniRead($ini,"ESLConfig","ecoserverport_game",3000))
	$ecoserverportweb = Int(IniRead($ini,"ESLConfig","ecoserverport_web",3001))
	$ecoserverurl = "http://127.0.0.1:" & $ecoserverportweb & "/"
	$logfile = IniRead($ini,"ESLConfig","logfile","log.txt")
	$logfilepath = @ScriptDir & "\" & $logfile
	;$interval_sec = Int(IniRead($ini,"ESLConfig","interval",10))
	$interval_sec = 10
	;Micro Interval (executed 10 times during one interval)
	$interval_ms = $interval_sec * 100
	$disablecheck = Int(IniRead($ini,"ESLConfig","disable",0))>0
	$backupmove = IniRead($ini,"ESLConfig","backupmove","")
	$backuporigin = $ecoserverpath & "Storage\Backup\"

	$inisavedata = @ScriptDir & "\param\savedata.ini"
	$publicIP = IniRead($inisavedata, "SAVEDATA", "LastPublicIP", $publicIP)
	$pid = IniRead($inisavedata, "SAVEDATA", "LastEcoServerPID", $pid)
	$lockcmd = Int(IniRead($ini,"ESLConfig","LockCMD",1))>0
	SetTrayState($trayCheck, $disablecheck)
	SetTrayState($trayCMDlock, $lockcmd)
EndFunc

Func EcoServerStartup()
	$inisavedata = @ScriptDir & "\param\savedata.ini"
	;See any Eco-Server running?
	$infocheck = False
	$ecoserverloadfinish = False
	$plist = ProcessList($ecoserverexe)
	If $plist[0][0]>0 Then
		;Seeing other Eco-Server
		$pid = $plist[1][1]
		LogMessage(1,"Seeing other EcoServer on PID " & $pid,"")
		IniWrite($inisavedata, "SAVEDATA", "LastEcoServerPID", $pid)
		Sleep(1000)
	EndIf
	If $pid<0 Then
		$pid = ShellExecute($ecoserverpath & $ecoserverexe, $parameters, $ecoserverpath)
		$timerstartup = TimerInit()
		LogMessage(1,"Started EcoServer on PID " & $pid,"")
		IniWrite($inisavedata, "SAVEDATA", "LastEcoServerPID", $pid)
		Sleep(1000)
	EndIf
	If $lockcmd Then EcoServerLockCMDWindow()
EndFunc

Func EcoServerShutdown()
	$inisavedata = @ScriptDir & "\param\savedata.ini"
	$plist = ProcessList($ecoserverexe)
	If $plist[0][0]>0 Then
		;Seeing other Eco-Server
		$pid = $plist[1][1]
	EndIf
	If $pid<>-1 Then
		$infocheck = False
		ProcessClose($pid)
		LogMessage(1,"Killed EcoServer on PID " & $pid,"")
		Sleep(1000)
	EndIf
EndFunc

Func EcoServerClose()
	$inisavedata = @ScriptDir & "\param\savedata.ini"
	$plist = ProcessList($ecoserverexe)
	If $plist[0][0]>0 Then
		;Seeing other Eco-Server
		$pid = $plist[1][1]
	EndIf
	If $pid<>-1 Then
		$infocheck = False
		$hwnd = _GetHwndFromPID($pid)
		WinClose($hwnd)
		LogMessage(1,"Closed EcoServer on PID " & $pid,"")
		Sleep(1000)
	EndIf
EndFunc

Func EcoServerLoadfinischCheck()
	If Not $ecoserverloadfinish Then
		If PortCheckInternal("127.0.0.1",$ecoserverportgame) Then
			$ecoserverloadfinish = True
			LogMessage(1,"Eco Server loading was finished - PID is " & $pid,"")
		EndIf
	EndIf
EndFunc

Func EcoServerInfoTest()
	;Check Server-Info 5 Times - If it get 5 Errors the Test did not pass
	$checkpass = False
	If PortCheckInternal("127.0.0.1",$ecoserverportweb) Then
		For $i = 1 to 5
			$webinfo = _INetGetSource($ecoserverurl & "info", True)
			If Not @error And @extended>0 Then
				$checkpass = True
				ExitLoop
			Else
				LogMessage(0,"Info-Test try failed","")
			EndIf
			Sleep(1000)
		Next
	Else
		LogMessage(0,"Info-Test Port not opened","")
	EndIf
	Return $checkpass
EndFunc

Func CheckHangEcoServer()
	;Just check after the info was seen first
	If Not EcoServerInfoTest() Then
		;Server may Hang - Shutting down and Up
		If EcoServerLoaded() Then
			LogMessage(2,"EcoServerHangTest Failed","The local webservice did not reply within 5 checks. Server may hang or was totaly overloaded. Server will be closed and restarted.")
			EcoServerShutdown()
			Sleep(10000)
			EcoServerStartup()
		EndIf
	EndIf
EndFunc

Func CheckRestartEcoServer()
	If $pid = -1 Then
		$opid = $pid
		EcoServerStartup()
		LogMessage(1,"EcoServer - Process started on PID " & $pid,"")
	EndIf
	If Not ProcessExists($pid) Then
		$opid = $pid
		$pid = -1
		EcoServerStartup()
		LogMessage(2,"EcoServer - Process does not exist on PID " & $opid,"Your server may have crashed. PID was not found. It was restarted on PID " & $pid)
	EndIf
EndFunc

Func CheckInternetIP()
	$serverIP = EcoServerIPTest()
	If $serverIP = "" Then
		;We could not get the IP - May be it is an Internet Error
		$CheckInternet = 0
		LogMessage(1,"Problem with getting the public IP - May be it is an Internet Error","")
	Else
		$CheckInternet = 1
		If $publicIP == "" Then
			$publicIP = $serverIP
			IniWrite($inisavedata, "SAVEDATA", "LastPublicIP", $publicIP)
			LogMessage(1,"Public IP is " & $publicIP & " - Make shure you have no dynamic IP - see config for dyndns for more info", "")
			SetDynDNS()
		Else
			If $serverIP <> $publicIP Then
				$publicIP = $serverIP
				IniWrite($inisavedata, "SAVEDATA", "LastPublicIP", $publicIP)
				LogMessage(2,"Public IP was changed to " & $publicIP, "Public IP has changed to " & $publicIP & " - Maybe you have a dynamic IP. You may need something like www.duckdns.org / www.noip.com")
				SetDynDNS()
			EndIf
		EndIf
	EndIf

	If EcoServerLoaded() Then
		;Just inform if EcoServer is running and loaded
		$LastCheckInternet = IniRead($inisavedata, "SAVEDATA", "CheckInternet", $CheckInternet)
		If $LastCheckInternet<>$CheckInternet Then
			If $CheckInternet=1 Then
				LogMessage(2,"Restored getting IP","Your Server could see the external IP again.")
			Else
				;seems absurd but if the SMTP server is in local net we can send a mail
				LogMessage(2,"Problem getting IP","Your server may have no Internet. Please take a look at it.")
			EndIf
		EndIf
		IniWrite($inisavedata, "SAVEDATA", "CheckInternet", $CheckInternet)
	EndIf
EndFunc

Func EcoServerIPTest()
	$ipcheckurl="http://ipecho.net/plain" ;You need it in plain text!
	;Check IP 5 Times - If it get 5 Errors the Test did not pass
	$checkpass = False
	For $i = 1 to 5
		$getIP = _INetGetSource($ipcheckurl,True)
		If Not @error And @extended>0 Then
			$checkpass = True
			ExitLoop
		Else
			LogMessage(0,"IP-Test try failed","")
			$getIP = ""
		EndIf
		Sleep(1000)
	Next
	Return $getIP
EndFunc

Func CheckEcoPorts()
	$pc = PortCheckExternal($publicIP,$ecoserverportweb)
	If $pc = 1 Then
		$CheckPortWeb = 1
	ElseIf $pc = 0 Then
		$CheckPortWeb = 0
	Else
		LogMessage(1,"Portcheck Problem - Website not reached - This can happen do not worry about it.", "")
		Return
	EndIf
	$pc = PortCheckExternal($publicIP,$ecoserverportgame)
	If $pc = 1 Then
		$CheckPortGame = 1
	ElseIf $pc = 0 Then
		$CheckPortGame = 0
	Else
		LogMessage(2,"Portcheck Problem - Website not reached", "")
		Return
	EndIf

	If EcoServerLoaded() Then
		;Just inform if EcoServer is running and loaded
		$LastCheckPortGame = IniRead($inisavedata, "SAVEDATA", "CheckPortGame", $CheckPortGame)
		If $LastCheckPortGame<>$CheckPortGame Then
			If $CheckPortGame=1 Then
				LogMessage(2,"Port for Game is ONLINE", "Port for Game is ONLINE")
			Else
				LogMessage(2,"Port for Game is OFFLINE", "Port for Game is OFFLINE")
			EndIf
		EndIf

		$LastCheckPortWeb = IniRead($inisavedata, "SAVEDATA", "CheckPortWeb", $CheckPortWeb)
		If $LastCheckPortWeb<>$CheckPortWeb Then
			If $CheckPortWeb=1 Then
				LogMessage(2,"Port for Web is ONLINE", "Port for Web is ONLINE")
			Else
				LogMessage(2,"Port for Web is OFFLINE", "Port for Web is OFFLINE")
			EndIf
		EndIf

		IniWrite($inisavedata, "SAVEDATA", "CheckPortWeb", $CheckPortWeb)
		IniWrite($inisavedata, "SAVEDATA", "CheckPortGame", $CheckPortGame)
	EndIf

EndFunc

Func PortCheckExternal($ip, $port)
	$checkurl="https://home.chackl.com/portcheck.php?ip=[IP]&port=[PORT]" ;You need it in plain text!
	$checkurl=StringReplace($checkurl,"[IP]",$ip)
	$checkurl=StringReplace($checkurl,"[PORT]",$port)
	;Check IP 5 Times - If it get 5 Errors the Test did not pass
	$checkpass = -1
	For $i = 1 to 5
		$res = _INetGetSource($checkurl,True)
		If Not @error Then
			$checkpass = $res
			ExitLoop
		Else
			LogMessage(1,"Port-Test try failed - IP: " & $ip & " - Port: " & $port ,"")
		EndIf
		Sleep(1000)
	Next
	Return $checkpass
EndFunc

Func PortCheckInternal($ip, $port)
	Opt('TCPTimeout', 500)
	TCPStartup()
	$iSocket = TCPConnect($ip, $port)
	If @error Then
		TCPShutdown()
		Return False
	Else
		TCPCloseSocket($iSocket)
		TCPShutdown()
		Return True
	EndIf
EndFunc

Func EcoServerBackupMove()
	If $backupmove<>"" Then
		$backuplist = _FileListToArray($backuporigin, "*", $FLTA_FOLDERS, False)
		If Not @error Then
			For $i = 1 To $backuplist[0]
				;Parse Date / time of Backup
				$a = StringSplit($backuplist[$i],"-",1)
				$backuptime = $a[2] & "/" & $a[3] & "/" & $a[4] & " " & StringReplace($a[5],".",":")
				$diff = _DateDiff("n", $backuptime, _NowCalc())
				If $diff > 20 Then
					$backuppath = $a[2] & "\" & $a[3] & "\" & $a[4] & "\" & $a[5] & "\" & $backuplist[$i]
					$src = $backuporigin & $backuplist[$i] & "\"
					$dst = $backupmove & $backuppath & "\"
					DirCreate($dst)
					DirCopy($src, $dst, 1)
					DirRemove($src, 1)
					LogMessage(1,"Moved Backup '" & $backuplist[$i] & "'","")
				EndIf
			Next
		EndIf
	EndIf
EndFunc

Func EcoServerLoaded()
	return $ecoserverloadfinish AND ProcessExists($pid)
EndFunc

Func startup()
	;Startup Init
	init()
	;Endless While Loop
	While True
		If $interval_count <= 0 Then
			$interval_count = 10
			;Call each 10th turn
			OnInterval()
		EndIf
		If $interval_count_low <= 0 Then
			$interval_count_low = 60
			;Call each 100th turn
			OffIntervalLow()
		EndIf
		$interval_count -= 1
		$interval_count_low -= 1
		OffIntervalHigh()

		Switch TrayGetMsg()
			Case $trayCMDlock
				If $lockcmd Then
					EcoServerUnlockCMDWindow()
				Else
					EcoServerLockCMDWindow()
				EndIf
			Case $trayCheck
				;TraySwitchCheck()
		EndSwitch

		;Sleep for Interval
		Sleep($interval_ms)
	WEnd
EndFunc

Func TrayEventCMDlock()
	$ini = @ScriptDir & "\param\config.ini"
	If $lockcmd Then
		EcoServerUnlockCMDWindow()
		IniWrite($ini,"ESLConfig","LockCMD",0)
		LogMessage(1,"Server CMD window was unlocked","")
	Else
		EcoServerLockCMDWindow()
		IniWrite($ini,"ESLConfig","LockCMD",1)
		LogMessage(1,"Server CMD window was locked","")
	EndIf
EndFunc

Func TrayEventCheck()
	$ini = @ScriptDir & "\param\config.ini"
	If $disablecheck Then
		IniWrite($ini,"ESLConfig","disable",0)
		LogMessage(1,"Checking is enabled","")
	Else
		IniWrite($ini,"ESLConfig","disable",1)
		LogMessage(1,"Checking is disabled","")
	EndIf
EndFunc

Func TrayEventStartup()
	EcoServerStartup()
	$disablecheck = True
	TrayEventCheck()
EndFunc

Func TrayEventShutdown()
	$res = MsgBox(4,"Eco Server Shutdown","Are you shure to quit the server? Be shure to save your game!!!")
	If $res = 6 Then
		$disablecheck = False
		TrayEventCheck()
		LogMessage(1,"Manual Shutdown was requested - Checking Disabled","")
		EcoServerClose()
		$inisavedata = @ScriptDir & "\param\savedata.ini"
		IniWrite($inisavedata, "SAVEDATA", "LastEcoServerPID", -1)
		$pid = -1
		Sleep(1000)
	EndIf
EndFunc

Func TrayEventEmergency()
	LogMessage(1,"Emergency Shutdown was requested","")
	EcoServerShutdown()
	Exit
EndFunc

Func TrayEventQuit()
	LogMessage(1,"Launcher Exit","")
	Exit
EndFunc

Func TrayEventQuitAll()
	$res = MsgBox(4,"Eco Server Shutdown","Are you shure to quit the server? Be shure to save your game!!!")
	If $res = 6 Then
		LogMessage(1,"Manual Shutdown was requested","")
		EcoServerClose()
		LogMessage(1,"Launcher Exit","")
		Exit
	EndIf
EndFunc

Func TrayEventTestmail()
	SendMailToAdmin("Testmail", "Tested you Mail Settings - If you see this you had success")
	LogMessage(1,"Testmail was sent if you enabled SMTP Mailing." & $pid,"")
EndFunc

Func OnInterval()
	;Reload Config
	initconfig()
	If Not $disablecheck Then
		LogMessage(0,"Start Check","")
		;ECO SERVER specific Checks

		;Check Server Online
		CheckRestartEcoServer()

		EcoServerLoadfinischCheck()
		;Check Server Hang
		If $ecoserverloadfinish Then CheckHangEcoServer()

		;Message if Server was not loaded
		If Not $ecoserverloadfinish Then LogMessage(0,"Did not see loading finished","")

		;Checking some other Things

		;Check Internet and public IP
		CheckInternetIP()

		;ECO SERVER Servicing

		;Move Backups
		EcoServerBackupMove()
		LogMessage(0,"Finished Check","")
	Else
		LogMessage(0,"Checking disabled in config","")
	EndIf
EndFunc

Func OffIntervalLow()
	;Check Ports from external
	If $ecoserverloadfinish Then CheckEcoPorts()
EndFunc

Func OffIntervalHigh()
	;This Function is called more often!! - Emergency Things only
	CheckExit()
EndFunc

Func CheckExit()
	$ini = @ScriptDir & "\param\config.ini"
	If Int(IniRead($ini,"ESLConfig","exit",0))>0 Then
		IniWrite($ini,"ESLConfig","exit",0)
		Exit
	EndIf
EndFunc

Func SendMailToAdmin($subject,$msg)
	$ini = @ScriptDir & "\param\mail.ini"
	If IniRead($ini,"MAIL","usemailservice",0)>0 Then
		$smtp = IniRead($ini,"MAIL","smtp","")
		$port = Int(IniRead($ini,"MAIL","port",0))
		$usessl = Int(IniRead($ini,"MAIL","usessl",0))
		$username = IniRead($ini,"MAIL","username","")
		$password = IniRead($ini,"MAIL","password","")
		$sendfrommail = IniRead($ini,"MAIL","sendfromMail","")
		$sendfromname = IniRead($ini,"MAIL","sendfromName","")
		$sendtomail = IniRead($ini,"MAIL","sendtoMail","")
		$mailsubject = IniRead($ini,"MAIL","prefixSubject","")
		If $mailsubject=="" Then
			$mailsubject = $subject
		Else
			$mailsubject = $mailsubject & " - " & $subject
		EndIf
		LogMessage(1,"Send Mail","")
		_INetSmtpMailCom($smtp, $sendfromname, $sendfrommail, $sendtomail, $mailsubject, $msg, "", "", "", "Normal", $username, $password, $port, $usessl)
	EndIf
EndFunc

Func SetDynDNS()
	$ini = @ScriptDir & "\param\dyndns.ini"
	If IniRead($ini,"DYNDNS","usedyndns",0)>0 Then
		$updateurl = IniRead($ini,"DYNDNS","updateurl","")
		If $updateurl<>"" Then
			$updateurl = StringReplace($updateurl, "[IP]", $publicIP)
			$res = _INetGetSource($updateurl, True)
			LogMessage(1,"DynDns update result: " & $res, "")
		EndIf
	EndIf
EndFunc

Func LogMessage($level, $short, $long)
	;Level 0: Info
	;Level 1: Warning
	;Level 2: Error
	$timestamp = _DateTimeFormat(_NowCalc(),5)
	If  $level = 0 Then
		ConsoleWrite("[" & $timestamp & "] " & $short & @CRLF)
	ElseIf $level = 1 Then
		ConsoleWrite("[" & $timestamp & "] " & $short & @CRLF)
		FileWriteLine($logfilepath,"[" & $timestamp & "] " & $short)
	ElseIf $level = 2 Then
		SendMailToAdmin("[" & $timestamp & "] " & $short, $long)
		ConsoleWrite("[" & $timestamp & "] " & $long & @CRLF)
		FileWriteLine($logfilepath,"[" & $timestamp & "] " & $short)
	EndIf
EndFunc

Func EcoServerLockCMDWindow()
	$hwnd = _GetHwndFromPID($pid)
	If $hwnd <> -1 Then
		WinSetState($hwnd, "", @SW_DISABLE)
		;WinSetState($hwnd, "", @SW_HIDE)
		$lockcmd = True
	EndIf
EndFunc

Func EcoServerUnlockCMDWindow()
	$hwnd = _GetHwndFromPID($pid)
	If $hwnd <> -1 Then
		WinSetState($hwnd, "", @SW_ENABLE)
		;WinSetState($hwnd, "", @SW_SHOW)
		$lockcmd = False
	EndIf
EndFunc

;Function for getting HWND from PID
Func _GetHwndFromPID($PID)
	$hWnd = -1
	$winlist = WinList()
	For $i = 1 To $winlist[0][0]
		If $winlist[$i][0] <> "" Then
			$iPID2 = WinGetProcess($winlist[$i][1])
			If $iPID2 = $PID Then
				$hWnd = $winlist[$i][1]
				If StringLower(WinGetTitle($hWnd))==StringLower($ecoserverpath & $ecoserverexe) Then ExitLoop
			EndIf
		EndIf
	Next
	Return $hWnd
EndFunc;==>_GetHwndFromPID

Func CreateTrayEvent($label, $callfunc)
	$tray = TrayCreateItem($label)
	TrayItemSetOnEvent($tray, $callfunc)
	Return $tray
EndFunc

Func SetTrayState($tray, $state)
	If $state Then
		TrayItemSetState($tray, 1)
	Else
		TrayItemSetState($tray, 4)
	EndIf
EndFunc

;Startup Programm
startup()